﻿B4R=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=1.19
@EndOfDesignText@
'File: EthServer.bas
'Based upon: https://www.b4x.com/android/forum/threads/ethernet-server-socket.67211/

Sub Process_Globals
	Private bc As ByteConverter
	Private Astream As AsyncStreams	
	Private eth_server As EthernetServerSocket

	'Server Port
	Private Const SERVER_PORT As Byte = 80	
	
	'Option disconnect with default 1 (=do disconnect after channel set)
	Private Const DEFAULT_DISCONNECT As Byte = 1
End Sub

'Init the server and start listening - do not use Log() here
Public Sub Start
	eth_server.Initialize(SERVER_PORT, "Server_NewConnection")
	eth_server.Listen
End Sub

'Handle new client connection - calls AStream_NewData to parse the HTTP command.
Private Sub Server_NewConnection (NewSocket As EthernetSocket)
	Astream.Initialize(NewSocket.Stream, "AStream_NewData", "AStream_Error")
End Sub

'Handle new data received. Need to parse the buffer starting with GET:
'[AStream_NewData]*START*GET /channel=1&value=127&disconnect=1 HTTP/1.1 Host: NNN.NNN.NNN.NNN User-Agent: Mozilla/5.0 (Windows NT 10.0;*END*
'The HTTP GET command is parsed to get the mandatory arguments channel,value and optional disconnect.
'If the channel,value data is ok, the dmx channel is set.
'There is always an HTTP response to the client either with the arguments channel... or [ERROR] if GET command is invalid
Private Sub AStream_NewData (Buffer() As Byte)
	'Log("[AStream_NewData]*START*", bc.StringFromBytes(Buffer),"*END*")

	'Check if there is data
	If Buffer.Length == 0 Then Return
	
	'Split the received data buffer using space to get the arguments starting with /channel
	For Each s() As Byte In bc.Split(Buffer, " ")
		'Does the string contain /channel
		If bc.IndexOf(s, "/channel") <> -1 Then

			'Get the arguments channel=N&value=N (start at index 1 to avoid the /)
			'channel=1&value=127&disconnect=0
			'Split the args by &
			Dim args() As Byte = bc.SubString(s, 1)

			'Define the arguments to be assigned
			Dim channel As Int = -1
			Dim value As Int = -1
			Dim disconnect As Int = DEFAULT_DISCONNECT

			'Split the arguments by & and loop over to assign the value
			'Example:/channel=1&value=255&disconnect=1
			Dim i As Int = 1
			For Each arg() As Byte In bc.split(args , "&")
				'Log("[AStream_NewData]arg=",arg)
				Select i
					Case 1
						channel = GetArgValue(arg)
					Case 2
						value = GetArgValue(arg)
					Case 3
						disconnect = GetArgValue(arg)
				End Select
				i = i + 1
			Next

			'Check if the mandatory arguments channel,value are ok
			If channel <> -1 And value <> -1 Then
				Log("[AStream_NewData]channel", channel, ",value=",value, ",disconnect=",disconnect)

				'Set the DMX channel
				Main.DMX_Set_Channel(channel, value)

				'Send HTTP response to the client
				Astream.Write("HTTP/1.1 200").Write(CRLF)
				Astream.Write("Content-Type: text/html").Write(CRLF).Write(CRLF)
				'Astream.Write("<script>setTimeout(function(){location.href=""http://192.168.NNN.NNN""} , 20000);</script>")
				Astream.Write("channel:").Write(channel.As(String).GetBytes())
				Astream.Write(",value:").Write(value.As(String).GetBytes())
				Astream.Write(",disconnect:").Write(disconnect.As(String).GetBytes())

			'Mandatory arguments are invalid
			Else
				Log("[ERROR]HTTP Command invalid. Check:", args)
				Astream.Write("HTTP/1.1 200").Write(CRLF)
				Astream.Write("Content-Type: text/html").Write(CRLF).Write(CRLF)
				Astream.Write("[ERROR]HTTP Command invalid. Check:").Write(args)
			End If
	
			'Close the connection if flag set
			If disconnect == 1 Then
				CallSubPlus("CloseConnection", 200, 0)
			End If

		End If
	Next
End Sub

Private Sub AStream_Error
	'Log("[AStream_Error]Disconnected")
	'Need to use listen
	eth_server.Listen
End Sub

Private Sub ConnectNetwork(unused As Byte)	'ignore
	Main.ConnectToNetwork
End Sub

Private Sub CloseConnection(unused As Byte)
	If eth_server.Socket.Connected Then
		Log("[DMX Server] Close Connection")
		eth_server.Socket.Stream.Flush
		eth_server.Socket.Close
		Main.eth_connected = False
	End If
End Sub

'Helpers

'Get the arg value as int from argument string, i.e. channel=1
'Returns int or -1 in case of error
Private Sub GetArgValue(arg() As Byte) As Int
	'Example: channel=1 or value=127
	'Log("[GetArgValue]",arg)

	'Split the arg using = as separator
	Dim valuestr As String
	Dim value As Int = -1
	Dim i As Int = 0
	For Each ba() As Byte In bc.split(arg , "=")
		'Get the value as array entry 2 (index 1)
		If i == 1 Then
			'Check if there is a value
			If ba.Length > 0 Then
				valuestr = bc.StringFromBytes(ba)
				value = valuestr.As(Int)				
			End If
		End If
		i = i + 1
	Next
	Return value
End Sub
